/*
    This file is part of the lin-kd-tree library.
    Copyright (C) 2009  Laurent Poirrier

    The lin-kd-tree library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, version 3 of the License.

    The lin-kd-tree library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __KDTREE_ARRAY_H
#define __KDTREE_ARRAY_H
#include <stdint.h>
#include "space.h"
#include "kd-tree.h"

/*
  The kd-tree array functions handle the array structure holding information
  about all objects.
*/

// *********************************************************************
// Array kd-tree structure
// *********************************************************************
#define KDTREE_SIZE	uint32_t

typedef struct {
	kdtree_node path;
	void *obj;
} kdtree_object;

typedef struct {
	KDTREE_SIZE nobj_tree, nobj_alloc;
	kdtree_object *tree;
} kdtree_array;

typedef void (*kdtree_collision_callback) (void *o1, void *o2);

// *********************************************************************
// Functions
// *********************************************************************
kdtree_array *kdtree_array_new();
void kdtree_array_free(kdtree_array *b);

void kdtree_clear(kdtree_array *b, KDTREE_SIZE base_size);
int  kdtree_add_object(kdtree_array *b, kdtree_iaabb *bb, void *obj);

void kdtree_collisions(kdtree_array *b, kdtree_collision_callback cb);

// Helper functions
int  kdtree_add_aabb(kdtree_array *b, kdtree_space *s,
	kdtree_aabb *bb, void *obj);
int  kdtree_add_sphere(kdtree_array *b, kdtree_space *s,
	kdtree_sphere *bs, void *obj);


#endif

