/*
    This file is part of the lin-kd-tree library.
    Copyright (C) 2009  Laurent Poirrier

    The lin-kd-tree library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, version 3 of the License.

    The lin-kd-tree library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __GEN_GRAPH_H
#define __GEN_GRAPH_H
#include "vec_math.h"

typedef struct graph_surface_t {
	vec3 *vertices;
	struct graph_edge_t {
		int v[2];
	} *edges;
	struct graph_face_t {
		int v[3];
		int e[3];
	} *faces;
	int nvertices, nedges, nfaces;
} graph_surface;

graph_surface *graph_alloc(int nv, int ne, int nf);
graph_surface *graph_split(graph_surface *orig);

graph_surface *graph_gen_sphere(int divlevel);
graph_surface *graph_gen_sheet(int divlevel);

vec3 *graph_face_normals(graph_surface *g);
vec3 *graph_vertex_normals(graph_surface *g);


#endif

